# -*- coding: utf-8 -*-
#
import boto3
import os

from .base import ObjectStorage


class S3Storage(ObjectStorage):
    def __init__(self, config):
        self.bucket = config.get("BUCKET", "jumpserver")
        self.region = config.get("REGION", None)
        self.access_key = config.get("ACCESS_KEY", None)
        self.secret_key = config.get("SECRET_KEY", None)
        self.endpoint = config.get("ENDPOINT", None)

        try:
            self.client = boto3.client(
                's3', region_name=self.region,
                aws_access_key_id=self.access_key,
                aws_secret_access_key=self.secret_key,
                endpoint_url=self.endpoint
            )
        except ValueError:
            pass

    def upload(self, src, target):
        try:
            self.client.upload_file(Filename=src, Bucket=self.bucket, Key=target)
            return True, None
        except Exception as e:
            return False, e

    def exists(self, path):
        try:
            self.client.head_object(Bucket=self.bucket, Key=path)
            return True
        except Exception as e:
            return False

    def download(self, src, target):
        try:
            os.makedirs(os.path.dirname(target), 0o755, exist_ok=True)
            self.client.download_file(self.bucket, src, target)
            return True, None
        except Exception as e:
            return False, e

    def delete(self, path):
        try:
            self.client.delete_object(Bucket=self.bucket, Key=path)
            return True, None
        except Exception as e:
            return False, e

    def generate_presigned_url(self, path, expire=3600):
        try:
            return self.client.generate_presigned_url(
                ClientMethod='get_object',
                Params={'Bucket': self.bucket, 'Key': path},
                ExpiresIn=expire,
                HttpMethod='GET'), None
        except Exception as e:
            return False, e

    def list_buckets(self):
        response = self.client.list_buckets()
        buckets = response.get('Buckets', [])
        result = [b['Name'] for b in buckets if b.get('Name')]
        return result

    @property
    def type(self):
        return 's3'



# -*- coding: utf-8 -*-
#
import os

from obs.client import ObsClient
from .base import ObjectStorage


class OBSStorage(ObjectStorage):
    def __init__(self, config):
        self.endpoint = config.get("ENDPOINT", None)
        self.bucket = config.get("BUCKET", None)
        self.access_key = config.get("ACCESS_KEY", None)
        self.secret_key = config.get("SECRET_KEY", None)
        if self.access_key and self.secret_key and self.endpoint:
            proxy_host = os.getenv("proxy_host")
            proxy_port = os.getenv("proxy_port")
            proxy_username = os.getenv("proxy_username")
            proxy_password = os.getenv("proxy_password")
            self.obsClient = ObsClient(access_key_id=self.access_key, secret_access_key=self.secret_key, server=self.endpoint, proxy_host=proxy_host, proxy_port=proxy_port, proxy_username=proxy_username, proxy_password=proxy_password)
        else:
            self.obsClient = None

    def upload(self, src, target):
        try:
            resp = self.obsClient.putFile(self.bucket, target, src)
            if resp.status < 300:
                return True, None
            else:
                return False, resp.reason
        except Exception as e:
            return False, e

    def exists(self, path):
        resp = self.obsClient.getObjectMetadata(self.bucket, path)
        if resp.status < 300:
            return True
        return False

    def delete(self, path):
        try:
            resp = self.obsClient.deleteObject(self.bucket, path)
            if resp.status < 300:
                return True, None
            else:
                return False, resp.reason
        except Exception as e:
            return False, e

    def download(self, src, target):
        try:
            os.makedirs(os.path.dirname(target), 0o755, exist_ok=True)
            resp = self.obsClient.getObject(self.bucket, src, target)
            if resp.status < 300:
                return True, None
            else:
                return False, resp.reason
        except Exception as e:
            return False, e

    def list_buckets(self):
        resp = self.obsClient.listBuckets()
        if resp.status < 300:
            return [b.name for b in resp.body.buckets]
        else:
            raise RuntimeError(resp.status, str(resp.reason))

    @property
    def type(self):
        return 'obs'



#!/usr/bin/python
#
import os
import re

spliter = re.compile('[>=<]+')

core_req_path = os.environ.get("CORE_REQ_FILE")
if not core_req_path:
    raise ValueError("No CORE_REQ_FILE env found")

with open(core_req_path) as f:
    core_reqs = f.readlines()

core_reqs_mapper = {}
for req in core_reqs:
    pkg_version = spliter.split(req.strip())
    core_reqs_mapper[pkg_version[0]] = req.strip()

fixed_reqs = []
with open('../requirements.txt') as f:
    reqs = f.readlines()

for req in reqs:
    pkg_version = spliter.split(req)
    pkg = pkg_version[0]

    if pkg in core_reqs_mapper:
        req = core_reqs_mapper[pkg]
    fixed_reqs.append(req.strip() + '\n')


with open('../requirements.txt', 'w') as f:
    f.writelines(fixed_reqs)




# -*- coding: utf-8 -*-
#
import os
from .base import ObjectStorage, LogStorage


class JMSReplayStorage(ObjectStorage):
    def __init__(self, config):
        self.client = config.get("SERVICE")

    def upload(self, src, target):
        session_id = os.path.basename(target).split('.')[0]
        ok = self.client.push_session_replay(src, session_id)
        return ok, None

    def delete(self, path):
        return False, Exception("Not support not")

    def exists(self, path):
        return False

    def download(self, src, target):
        return False, Exception("Not support not")

    @property
    def type(self):
        return 'jms'


class JMSCommandStorage(LogStorage):
    def __init__(self, config):
        self.client = config.get("SERVICE")
        if not self.client:
            raise Exception("Not found app service")

    def save(self, command):
        return self.client.push_session_command([command])

    def bulk_save(self, command_set, raise_on_error=True):
        return self.client.push_session_command(command_set)

    def filter(self, date_from=None, date_to=None,
               user=None, asset=None, account=None,
               input=None, session=None):
        pass

    def count(self, date_from=None, date_to=None,
              user=None, asset=None, account=None,
              input=None, session=None):
        pass
